(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_structural_mapping.ml,v 1.19 2013-12-02 15:28:38 fclement Exp $ *)

(* Matrix structural mappings. *)

module Nat = Unatural;;
module Dgraph = Udynamic_graph;;
module Ddgraph = Udelayed_dgraph;;

module Idx = Uindex_index;;
module Uidx = Uindex_universe;;
module Set = Umatrix_nonempty_set;;
module Mapping = Umatrix_graph;;

type 'a mapping = 'a Mapping.t;;

type block_number = Ddgraph.arrow_number;;

type ('a, 'b) block = ('a mapping, 'b) Ddgraph.delayed_arrow;;

type ('a, 'b, 'c) t = ('a mapping, 'b, 'c) Ddgraph.t;;

let same = Ddgraph.same;;

(* Access functions. *)

let get = Ddgraph.get
and set = Ddgraph.set
and set_next = Ddgraph.set_next
;;

let last_block_number msm = Nat.pred (Dgraph.next_unset msm.Ddgraph.graph);;

(* Converters. *)

let full_domain_of_msm msm =
  let { Dgraph.value = map; _; } = get msm Nat._0_ in
  let u = map.Mapping.domain.Set.universe in
  Set.full u
;;

let msm_choice msm =
  let { Dgraph.value = map; _; } = get msm Nat._0_ in
  match map.Mapping.images with
  | Mapping.Constant y -> y
  | Mapping.Images ims -> ims.(0)
;;

let images_of_msm msm dom =
  let y0 = msm_choice msm in
  let ims = Uarray.make (Set.cardinal dom) y0 in
  let f_item _i { Dgraph.value = item_map; _; } =
    match item_map with
    | { Mapping.domain = item_dom;
        images = Mapping.Constant item_y; } ->
        let f _ item_uidx =
          let idx = Uidx.index dom.Set.uindices item_uidx in
          let i = Idx.to_int idx in
          ims.(i) <- item_y in
        Set.iter f item_dom
    | { Mapping.domain = item_dom;
        images = Mapping.Images item_ims; } ->
        let f item_idx item_uidx =
          let idx = Uidx.index dom.Set.uindices item_uidx in
          let i = Idx.to_int idx in
          let item_i = Idx.to_int item_idx in
          ims.(i) <- item_ims.(item_i) in
        Set.iter f item_dom in
  Dgraph.iter f_item msm.Ddgraph.graph;
  ims
;;

let to_graph msm =
  let dom = full_domain_of_msm msm in
  let ims = images_of_msm msm dom in
  Mapping.from_array dom ims
;;

let of_graph f = Ddgraph.make f Nat._1_;;

(* Iterators. *)

let iter = Ddgraph.iter;;

let fold_left = Ddgraph.fold_left;;

let iter_to_do = Ddgraph.iter_to_do;;

let iter_value = Ddgraph.iter_value;;

let fold_left_value = Ddgraph.fold_left_value;;
