(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ulist1_zipper.ml,v 1.1 2013/06/21 17:31:34 fclement Exp $ *)

let lz_failwith = Upervasives.failwith "Ulist1_zipper";;

type 'a location =
  {
   before : 'a list;
   here : 'a Ulist1.t;
  }
;;

let top l1 = { before = []; here = l1; };;

let rec zip { before = before; here = here; } =
  match before with
  | [] -> here
  | x :: l -> zip { before = l; here = Ulist1.cons x here; }
;;

let unzip loc_b l1_a =
  let rec loop = function
    | ([], loc_a) -> loc_a
    | (_ :: l_b, { before = before; here = (a0, a1 :: l_a); }) ->
        let loc_a = { before = a0 :: before; here = (a1, l_a); } in
        loop (l_b, loc_a)
    | (_ :: _, { here = (_, []); _; }) ->
        lz_failwith "unzip" "no such location" in
  loop (loc_b.before, (top l1_a))
;;

let rec fold_left f accu = function
  | { here = (x, []); _; } as loc -> f accu loc x
  | { before = before; here = (x, y :: l); } as loc ->
      let accu = f accu loc x in
      fold_left f accu { before = x :: before; here = (y, l); }
;;
